# Copyright 2021-2023 Datum Technology Corporation
# SPDX-License-Identifier: GPL-3.0
########################################################################################################################


import jinja2
from jinja2 import Template
from mio import common
from mio import cfg
from mio import user
import re
import os
from datetime import date


uvm_gen_dir               = re.sub("init.py", "", os.path.realpath(__file__)) + ".."
relative_path_to_template = uvm_gen_dir + "/templates/"
project_toml_file_path    = relative_path_to_template + "mio.toml"
dv_ip_file_path           = relative_path_to_template + "dv_ip.yml"
rtl_ip_file_path          = relative_path_to_template + "rtl_ip.yml"
vivado_ip_file_path       = relative_path_to_template + "vivado_ip.yml"
ts_yml_file_path          = relative_path_to_template + "ts.yml.j2"


def new_project(wd):
    common.dbg(f"Using {wd} as working directory")
    common.banner("Welcome to the Moore.io Project Initializer v1.0")
    project_name = ""
    project_full_name = ""
    copyright_owner = ""
    while project_name == "":
        project_name = common.prompt("Please enter the project name (ex: 'example_chip'):").strip().lower()
    while project_full_name == "":
        project_full_name = common.prompt("Please enter the full project name (ex: 'Example Chip'):").strip()
    copyright_owner = user.org_full_name
    try:
        fin = open(project_toml_file_path, "rt")
        data = fin.read()
        template = Template(data)
        data = template.render(name=project_name, full_name=project_full_name, year=date.today().year, name_of_copyright_owner=copyright_owner)
        fout_path = wd + "/mio.toml"
        fout = open(fout_path, "w+")
        common.dbg(f"Writing to {fout_path}")
        fout.write(data)
        fout.close()
        common.create_dir (wd + "/dft"  )
        common.create_dir (wd + "/docs" )
        common.create_dir (wd + "/dv"   )
        common.create_dir (wd + "/lint" )
        common.create_dir (wd + "/rtl"  )
        common.create_dir (wd + "/sim"  )
        common.create_dir (wd + "/syn"  )
        common.create_dir (wd + "/tools")
        
        common.copy_file(relative_path_to_template + "/project_gitignore", wd + "/.gitignore")
        common.copy_file(relative_path_to_template + "/sim_gitignore"    , wd + "/sim/.gitignore")
    except Exception as e:
        common.fatal("Failed to create project structure: " + str(e))
    common.info("Project created successfully.")


def new_ip(wd):
    common.dbg(f"Using {wd} as working directory")
    if os.path.exists(f"{wd}/ip.yml"):
        common.fatal(f"Cannot initialize existing IP: '{wd}/ip.yml'")
    
    common.info("Welcome to the Moore.io IP Initializer v1.0")
    common.info("  The following IP types are available:")
    common.info("  [0] - RTL IP")
    common.info("  [1] - Vivado Project IP")
    common.info("  [2] - DV  IP")
    ip_type = -1
    while ip_type != 0 and ip_type != 1 and ip_type != 2:
        ip_type = common.prompt("Please select the type of IP:").strip().lower()
        ip_type = int(ip_type)
    ip_name = ""
    ip_full_name = ""
    copyright_owner = ""
    while ip_name == "":
        ip_name = common.prompt("Please enter the IP name (ex: 'example_ip'):").strip().lower()
    while ip_full_name == "":
        ip_full_name = common.prompt("Please enter the full IP name (ex: 'Example IP'):").strip()
    
    # TODO Improvement: Ask the user for these
    bin_dir  = "bin"
    docs_dir = "docs"
    ex_dir   = "examples"
    src_dir  = "src"
    
    if ip_type == 2:
        create_ip_dirs(wd, bin_dir, docs_dir, ex_dir, src_dir)
        data = new_dv_ip(wd, ip_name, ip_full_name, bin_dir, docs_dir, ex_dir, src_dir)
        try:
            fout_path = wd + "/ip.yml"
            fout = open(fout_path, "w+")
            common.dbg(f"Writing to {fout_path}")
            fout.write(data[0])
            fout.close()
            fout_path = wd + "/" + src_dir + "/ts.yml"
            fout = open(fout_path, "w+")
            common.dbg(f"Writing to {fout_path}")
            fout.write(data[1])
            fout.close()
        except Exception as e:
            common.fatal("Failed to create IP structure: " + str(e))
    else:
        if ip_type == 0:
            create_ip_dirs(wd, bin_dir, docs_dir, ex_dir, src_dir)
            data = new_rtl_ip(wd, ip_name, ip_full_name, bin_dir, docs_dir, ex_dir, src_dir)
        elif ip_type == 1:
            src_dir  = "xsim"
            data = new_viv_ip(wd, ip_name, ip_full_name, bin_dir, docs_dir, ex_dir, src_dir)
        try:
            fout_path = wd + "/ip.yml"
            fout = open(fout_path, "w+")
            common.dbg(f"Writing to {fout_path}")
            fout.write(data)
            fout.close()
        except Exception as e:
            common.fatal("Failed to create IP structure: " + str(e))
    common.info(f"IP {ip_name} initialized successfully.")


def new_dv_ip(wd, name, full_name, bin_dir, docs_dir, ex_dir, src_dir):
    copyright_owner = user.org_full_name
    vendor = user.org_name.lower()
    has_dut = False
    top_file = ""
    top_construct = ""
    dut_name = ""
    fsoc = False
    fsoc_dut_full_name = ""
    fsoc_dut_target = ""
    
    while top_file == "":
        top_file = common.prompt(f"Please enter the top file name (ex: '{name}_pkg.sv'):").strip()
    
    is_tb = ""
    while is_tb not in ["yes", "y", "no", "n"]:
        is_tb = common.prompt("Is this a Test Bench? (y/n)").strip().lower()
    if (is_tb == "y") or (is_tb == "yes"):
        has_dut = True
    
    dut_fsoc  = ""
    if has_dut:
        while dut_fsoc not in ["yes", "y", "no", "n"]:
            dut_fsoc = common.prompt("Is the DUT a FuseSoC Core? (y/n)").strip().lower()
        if (dut_fsoc == "y") or (dut_fsoc == "yes"):
            fsoc = True
            while dut_name == "":
                dut_name = common.prompt("Please enter the DUT FuseSoC Core name (ex: 'core-v-mcu'):").strip().lower()
            while fsoc_dut_full_name == "":
                fsoc_dut_full_name = common.prompt("Please enter the DUT FuseSoC Core fully defined name (ex: 'openhwgroup.org:systems:core-v-mcu'):").strip().lower()
            while fsoc_dut_target == "":
                fsoc_dut_target = common.prompt("Please enter the DUT FuseSoC Core target name (ex: 'xsim'):").strip().lower()
        else:
            while dut_name == "":
                dut_name = common.prompt("Please enter the DUT IP name (ex: '@acme/example'):").strip().lower()
        while top_construct == "":
            top_construct = common.prompt(f"Please enter the Test Bench's top module name (ex: '{name}_tb'):").strip()
    
    template_args = {
        "simulator"               : common.get_simulator_short_name(cfg.default_simulator),
        "bin_dir"                 : bin_dir           ,
        "docs_dir"                : docs_dir          ,
        "ex_dir"                  : ex_dir            ,
        "src_dir"                 : src_dir           ,
        "name"                    : name              ,
        "vendor"                  : vendor            ,
        "full_name"               : full_name         ,
        "has_dut"                 : has_dut           ,
        "top_file"                : top_file          ,
        "top_construct"           : top_construct     ,
        "dut_name"                : dut_name          ,
        "fsoc"                    : fsoc              ,
        "fsoc_dut_full_name"      : fsoc_dut_full_name,
        "fsoc_dut_target"         : fsoc_dut_target   ,
        "year"                    : date.today().year ,
        "name_of_copyright_owner" : copyright_owner
        }
    try:
        fin = open(dv_ip_file_path, "r")
        data = fin.read()
        template = Template(data)
        ip_yml = template.render(template_args)
        fin = open(ts_yml_file_path, "r")
        data = fin.read()
        template = Template(data)
        ts_yml = template.render(template_args)
    except Exception as e:
        common.fatal(f"Failed to create ip.yml at {wd}: {e}")
    return [ip_yml, ts_yml]


def new_rtl_ip(wd, name, full_name, bin_dir, docs_dir, ex_dir, src_dir):
    copyright_owner = user.org_full_name
    vendor = user.org_name.lower()
    
    top_file = ""
    while top_file == "":
        top_file = common.prompt("Please enter the top file name (ex: 'top.sv'):").strip()
    
    template_args = {
        "simulator"               : common.get_simulator_short_name(cfg.default_simulator),
        "bin_dir"                 : bin_dir          ,
        "docs_dir"                : docs_dir         ,
        "ex_dir"                  : ex_dir           ,
        "src_dir"                 : src_dir          ,
        "name"                    : name             ,
        "vendor"                  : vendor           ,
        "full_name"               : full_name        ,
        "top_file"                : top_file         ,
        "year"                    : date.today().year,
        "name_of_copyright_owner" : copyright_owner
        }
    try:
        fin = open(rtl_ip_file_path, "r")
        data = fin.read()
        template = Template(data)
    except Exception as e:
        common.fatal(f"Failed to create ip.yml at {wd}: {e}")
    return template.render(template_args)


def new_viv_ip(wd, name, full_name, bin_dir, docs_dir, ex_dir, src_dir):
    copyright_owner = user.org_full_name
    vendor = user.org_name.lower()
    
    lib_name = ""
    while lib_name != "":
        lib_name = common.prompt("Please enter the Vivado Library Name:").strip()
    
    template_args = {
        "bin_dir"                 : bin_dir          ,
        "docs_dir"                : docs_dir         ,
        "ex_dir"                  : ex_dir           ,
        "src_dir"                 : src_dir          ,
        "name"                    : name             ,
        "vendor"                  : vendor           ,
        "full_name"               : full_name        ,
        "lib_name"                : lib_name         ,
        "year"                    : date.today().year,
        "name_of_copyright_owner" : copyright_owner
        }
    try:
        fin = open(vivado_ip_file_path, "r")
        data = fin.read()
        template = Template(data)
    except Exception as e:
        common.fatal(f"Failed to create ip.yml at {wd}: {e}")
    return template.render(template_args)


def create_ip_dirs(wd, bin_dir, docs_dir, ex_dir, src_dir):
    common.create_dir(wd + f"/{bin_dir}" )
    common.create_dir(wd + f"/{docs_dir}")
    common.create_dir(wd + f"/{ex_dir}"  )
    common.create_dir(wd + f"/{src_dir}" )



# Copyright 2021-2023 Datum Technology Corporation
# SPDX-License-Identifier: GPL-3.0
########################################################################################################################

from mio import common
from mio import cache
from mio import cfg
from mio import eal
import os

import subprocess


def gen_doxygen(ip_str, inc_deps=True, inc_tags=True):
    vendor, name = common.parse_dep(ip_str)
    if vendor == "":
        ip = cache.get_anon_ip(name, True)
    else:
        ip = cache.get_ip(vendor, name, True)
    ip_name = f"{ip.vendor}/{ip.name}"
    common.dbg(f"Running doxygen on '{ip_name}'")
    ip_src_path      = ip.path + "/" + ip.src_path
    ip_bin_path      = ip.path + "/" + ip.scripts_path
    ip_docs_path     = ip.path + "/" + ip.docs_path
    ip_examples_path = ip.path + "/" + ip.examples_path
    
    dep_example_paths = ""
    dep_src_paths     = ""
    dep_docs_paths    = ""
    
    common.create_dir(f"{ip_docs_path}")
    common.create_dir(f"{ip_docs_path}/dox_out")
    common.create_dir(f"{ip_docs_path}/dox_out/html")
    
    if inc_deps:
        # Local code
        if ip.has_dut:
            if ip.dut != None:
                dut_ip = ip.dut.target_ip_model
                if (not dut_ip.from_ip_server) and (not dut_ip.is_encrypted):
                    dep_example_paths += f"{dut_ip.path}/{dut_ip.examples_path} "
                    dep_src_paths     += f"{dut_ip.path}/{dut_ip.src_path} "
                    dep_docs_paths    += f"{dut_ip.path}/{dut_ip.docs_path} "
        deps = ip.get_ordered_deps()
        for dep in deps:
            if (not dep.from_ip_server) and (not dep.is_encrypted):
                dep_example_paths += f"{dep.path}/{dep.examples_path} "
                dep_src_paths     += f"{dep.path}/{dep.src_path} "
                dep_docs_paths    += f"{dep.path}/{dep.docs_path} "
    
    args  = " PROJECT_NAME='"  + ip.full_name + "'"
    args += " PROJECT_BRIEF='" + ip.full_name + "'"
    args += " IP_NAME='" + ip.name + "'"
    args += " PROJECT_NUMBER='" + ip.name + "'"
    args += " EXAMPLE_PATH=" + os.path.relpath(ip_examples_path, ip.path)
    args += " OUTPUT_PATH="  + os.path.relpath(ip_docs_path, ip.path) + "/dox_out"
    args += " SRC_PATH='" + os.path.relpath(ip_src_path, ip.path) + f" {dep_src_paths}'"
    args += " MIO_HOME=" + cfg.mio_data_src_dir + " IP_NAME=" + ip_name
    args += " DOCS_PATH+=" + os.path.relpath(ip_docs_path, ip.path)
    
    ip_docs_path = os.path.relpath(ip_docs_path, ip.path)
    args += f" GENERATE_TAGFILE='{ip_docs_path}/dox_out/{ip.vendor}__{ip.name}.tag'"
    if inc_tags:
        tag_files = ""
        if ip.has_dut:
            dut_ip = ip.dut.target_ip_model
            if dut_ip.from_ip_server:
                tag_files += f"{dut_ip.path}/{dut_ip.docs_path}/dox_out/{dut_ip.vendor}__{dut_ip.name}.tag={dut_ip.path}/{dut_ip.docs_path}/dox_out/html "
        deps = ip.get_ordered_deps()
        for dep in deps:
            if dep.from_ip_server and (dep.name != "uvml_mio_lic"):
                tag_files += f"{dep.path}/{dep.docs_path}/dox_out/{dep.vendor}__{dep.name}.tag={dep.path}/{dep.docs_path}/dox_out/html "
        if tag_files != "":
            args += f" TAGFILES='{tag_files}'"
    
    #args += " IMAGE_PATH=" + ip_path + "/" + ip_metadata['structure']['docs-path']
    #args += " INPUT+="     + ip_path + "/" + ip_metadata['structure']['docs-path']
    #for input_dir in ip_metadata['hdl-src']['directories']:
    #    if input_dir != "." and input_dir != "":
    #        args += " INPUT+=" + ip_src_path + "/" + input_dir
    #    else:
    #        args += " INPUT+=" + ip_src_path
    common.banner(f"Invoking Doxygen on IP '{ip_name}' ({ip_src_path})")
    eal.launch_eda_bin(args + " doxygen", [cfg.mio_data_src_dir + "/doxygen.private.cfg"], wd=ip.path, output=cfg.dbg)
    common.info("Done.  To view documentation: `firefox " + ip.path + "/" + ip.docs_path + "/dox_out/html/index.html &`")



#!/usr/bin/python3
########################################################################################################################
# Copyright 2021-2023 Datum Technology Corporation
# All rights reserved
########################################################################################################################


########################################################################################################################
# IMPORTS
########################################################################################################################
from datetime import date
import os
import sys
import re
import jinja2
from jinja2 import Template
from mio import common
from mio import new
from mio import cfg
#from vsdx import VisioFile


########################################################################################################################
# GLOBALS
########################################################################################################################
dbg = False
uvm_gen_dir = re.sub("new_agent_serial.py", "", os.path.realpath(__file__)) + ".."
relative_path_to_template = uvm_gen_dir + "/templates/sets/"
out_path = ""
default_license = "All rights reserved."
name_of_copyright_owner = ""
name = ""
full_name = ""


########################################################################################################################
# TEMPLATE DATA
########################################################################################################################
parameters = { }


########################################################################################################################
# MAIN
########################################################################################################################
def combine_dict(d1, d2):
    d3 = {}
    for d in d1:
        d3[d] = d1[d]
    for d in d2:
        d3[d] = d2[d]
    return d3


def pick_out_path():
    global out_path
    global default_copyright_owner
    if len(sys.argv) > 1:
        out_path = sys.argv[1]
        print("Code will be output to " + out_path)
    if len(sys.argv) > 2:
        default_copyright_owner = sys.argv[2].replace('"', "")
        print("Default copyright owner is " + default_copyright_owner)


def prompt_user_values():
    global out_path
    global name
    global full_name
    global name_of_copyright_owner
    global parameters
    
    default_copyright_owner = cfg.org_full_name
    default_vendor          = cfg.org_name
    
    common.info("Moore.io Template Generator: UVM Advanced Serial Agent - (v1p0)")
    common.info("***************************************************************")
    common.info("The answers to the following questionnaire will be used to generate the code for your new UVM Agent")
    common.info("")
    
    if out_path == "":
        out_path = common.prompt("Please enter the destination path for this new agent (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    
    license = common.prompt("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    
    name = common.prompt("Please enter the package name for this agent (ex: 'spi'); this name will be used for all agent types (ex: 'uvma_spi_agent_c'):\n").lower().strip()
    if name == "":
        sys.exit("ERROR: package name cannot be empty.  Exiting.")
    
    full_name = common.prompt("Please enter the full name for this agent (ex: 'Serial Peripheral Interface'):\n").strip()
    if full_name == "":
        sys.exit("ERROR: full name cannot be empty.  Exiting.")
    
    is_symmetric_str = common.prompt("Is the physical interface symmetric?  [N/y]").strip().lower()
    if is_symmetric_str == "":
        is_symmetric = False
    else:
        if is_symmetric_str == "n" or is_symmetric_str == "no":
            is_symmetric = False
        elif is_symmetric_str == "y" or is_symmetric_str == "yes":
            is_symmetric = True
        else:
            sys.exit("ERROR: please enter 'y' or 'n'")
    
    is_ddr_str = common.prompt("Is the physical interface using DDR clocking?  [N/y]").strip().lower()
    if is_ddr_str == "":
        is_ddr = False
    else:
        if is_ddr_str == "n" or is_ddr_str == "no":
            is_ddr = False
        elif is_ddr_str == "y" or is_ddr_str == "yes":
            is_ddr = True
        else:
            sys.exit("ERROR: please enter 'y' or 'n'")
    
    mode_1 = common.prompt("Please enter the first mode for this new agent (default: 'mstr'):\n").strip()
    if mode_1 == "":
        mode_1 = "mstr"
    
    mode_2 = common.prompt("Please enter the second mode for this new agent (default: 'slv'):\n").strip()
    if mode_2 == "":
        mode_2 = "slv"
    
    tx_str = common.prompt("Please enter the first direction for this new agent (default: 'm2s'):\n").strip()
    if tx_str == "":
        tx_str = "m2s"
    
    rx_str = common.prompt("Please enter the second direction for this new agent (default: 's2m'):\n").strip()
    if rx_str == "":
        rx_str = "s2m"
    
    parameters = {
        "name"                    : name,
        "full_name"               : full_name,
        "name_of_copyright_owner" : name_of_copyright_owner,
        "license"                 : license,
        "year"                    : date.today().year,
        "symmetric"               : is_symmetric,
        "ddr"                     : is_ddr,
        "mode_1"                  : mode_1,
        "mode_2"                  : mode_2,
        "tx"                      : tx_str,
        "rx"                      : rx_str,
        "dmp0"                    : "drv_" + tx_str + "_mp",
        "dmp1"                    : "drv_" + rx_str + "_mp",
        "vendor"                  : name_of_vendor
    }


def gen_agent():
    common.fatal("Not yet implemented")


def print_end_message():
    global out_path
    global name
    global parameters
    
    common.banner("IPs successfully generated:")
    common.info("  * " + out_path + "/uvma_" + name)
    common.info("  * " + out_path + "/uvme_" + name + "_st")
    common.info("  * " + out_path + "/uvmt_" + name + "_st")


########################################################################################################################
# ENTRY POINT
########################################################################################################################
def main():
    #pick_out_path()
    prompt_user_values()
    gen_agent()
    print_end_message()



# Copyright 2021-2023 Datum Technology Corporation
# SPDX-License-Identifier: GPL-3.0
########################################################################################################################


from mio import cache
from mio import common
from mio import cfg
from mio import clean
from mio import cov
from mio import dox
from mio import sim

import math
import json
import yaml
from yaml.loader import SafeLoader
import os
import jinja2
from jinja2 import Template
import xml.etree.cElementTree as ET
from datetime import datetime
import re


uvm_gen_dir = re.sub("results.py", "", os.path.realpath(__file__)) + ".."
relative_path_to_template = uvm_gen_dir + "/templates/"

interactive_html_report_template_path = relative_path_to_template + "interactive_results.html.j2"
regression_html_report_template_path  = relative_path_to_template + "regression_results.html.j2"


class Results:
    """results model"""

    def __init__(self, ip, name, is_regression, timestamp):
        self.name                        = name
        self.timestamp                   = timestamp
        self.is_regression               = is_regression
        self.simulator                   = ""
        self.regression_name             = ""
        self.regression_timestamp_start  = ""
        self.regression_timestamp_end    = ""
        self.regression_duration         = ""
        self.regression_duration_str     = ""
        self.default_test_suite          = True
        self.test_suite_name             = ""
        self.ip_str                      = ""
        self.ip_vendor                   = ip.vendor
        self.ip_name                     = ip.name
        self.target_name                 = ""
        self.has_target                  = False
        self.passed                      = False
        self.duration                    = 0
        self.duration_str                = ""
        self.num_tests                   = -1
        self.num_sets                    = -1
        self.num_groups                  = -1
        self.num_failed_tests            = -1
        self.num_passed_tests            = -1
        self.pct_passed                  = -1
        self.pct_failed                  = -1
        self.html_report_path            = ""
        self.xml_report_path             = ""
        self.failing_tests               = []
        self.sets                        = {}
    
    def add_set(self, set):
        set.results = self
        self.sets[set.name] = set
    
    def render(self):
        for set in self.sets:
            for group in self.sets[set].groups:
                for test in self.sets[set].groups[group].passing_tests:
                    test.render()
                for test in self.sets[set].groups[group].failing_tests:
                    test.render()
                    self.failing_tests.append(test)
                self.sets[set].groups[group].render()
            self.sets[set].render()
        
        self.num_sets = len(self.sets)
        self.duration = 0
        self.num_groups = 0
        self.num_tests = 0
        self.num_passed_tests = 0
        self.num_failed_tests = 0
        for set in self.sets:
            self.duration         += self.sets[set].duration
            self.num_groups       += self.sets[set].num_groups
            self.num_tests        += self.sets[set].num_tests
            self.num_passed_tests += self.sets[set].num_passed_tests
            self.num_failed_tests += self.sets[set].num_failed_tests
        self.pct_passed = int(self.num_passed_tests / (self.num_passed_tests + self.num_failed_tests) * 100)
        self.pct_failed = 100 - self.pct_passed
        if self.pct_passed == 100:
            self.passed = True
        if self.target_name != "default":
            has_target = True
        if self.is_regression and self.has_target:
            self.ip_str = f"{self.ip_name}#{self.target_name}"
        else:
            self.ip_str = f"{self.ip_name}"
        if self.is_regression:
            start = datetime.strptime(self.regression_timestamp_start, "%Y_%m_%d_%H_%M_%S")
            end   = datetime.strptime(self.regression_timestamp_end  , "%Y/%m/%d-%H:%M:%S")
            duration = end - start
            self.regression_duration = divmod(duration.seconds, 60)[1]
            self.regression_timestamp_start = start.strftime("%Y/%m/%d-%H:%M:%S")
            hours = math.floor(self.regression_duration / 3600)
            minutes = math.floor((self.regression_duration - (hours*3600)) / 60)
            seconds = math.ceil(self.regression_duration - (hours*3600) - (minutes*60))
            self.regression_duration_str = f"{str(hours)} hour(s), {str(minutes)} minute(s), {str(seconds)} second(s)"
        for set in self.sets:
            self.sets[set].duration_pct = int(self.sets[set].duration / self.duration * 100)
        hours = math.floor(self.duration / 3600)
        minutes = math.floor((self.duration - (hours*3600)) / 60)
        seconds = math.ceil(self.duration - (hours*3600) - (minutes*60))
        self.duration_str = f"{str(hours)} hour(s), {str(minutes)} minute(s), {str(seconds)} second(s)"


class TestSet:
    def __init__(self, name):
        self.name = name
        self.results = None
        self.duration = -1
        self.duration_pct = -1
        self.num_groups = -1
        self.num_tests = -1
        self.pct_passed = -1
        self.pct_failed = -1
        self.num_passed_tests = -1
        self.num_failed_tests = -1
        self.passed = False
        self.groups = {}
    
    def add_group(self, group):
        group.set = self
        self.groups[group.name] = group
    
    def render(self):
        self.num_groups = len(self.groups)
        self.duration = 0
        self.num_tests = 0
        self.num_passed_tests = 0
        self.num_failed_tests = 0
        for group in self.groups:
            self.duration         += self.groups[group].duration
            self.num_tests        += self.groups[group].num_tests
            self.num_passed_tests += self.groups[group].num_passed_tests
            self.num_failed_tests += self.groups[group].num_failed_tests
        self.pct_passed = int(self.num_passed_tests / (self.num_passed_tests + self.num_failed_tests) * 100)
        self.pct_failed = 100 - self.pct_passed
        if self.pct_passed == 100:
            self.passed = True
        for group in self.groups:
            self.groups[group].duration_pct = int(self.groups[group].duration / self.duration * 100)


class TestGroup:
    def __init__(self, name):
        self.name = name
        self.set = None
        self.duration = -1
        self.duration_pct = -1
        self.num_tests = -1
        self.pct_passed = -1
        self.pct_failed = -1
        self.num_passed_tests = -1
        self.num_failed_tests = -1
        self.passed = False
        self.passing_tests = []
        self.failing_tests = []
    
    def add_test(self, test):
        test.group = self
        test.set   = self.set
        if test.passed:
            self.passing_tests.append(test)
        else:
            self.failing_tests.append(test)
    
    def render(self):
        self.duration = 0
        self.num_passed_tests = len(self.passing_tests)
        self.num_failed_tests = len(self.failing_tests)
        self.num_tests  = self.num_passed_tests + self.num_failed_tests
        self.pct_passed = int(self.num_passed_tests / (self.num_passed_tests + self.num_failed_tests) * 100)
        self.pct_failed = 100 - self.pct_passed
        if self.pct_passed == 100:
            self.passed = True
        for test in self.passing_tests:
            self.duration += test.duration
        for test in self.failing_tests:
            self.duration += test.duration
        for test in self.passing_tests:
            test.duration_pct = int(test.duration / self.duration * 100)


class Test:
    def __init__(self, name, results_dir, results_path, sim_log_path, seed, duration):
        self.name         = name
        self.results_dir  = results_dir
        self.results_path = results_path
        self.sim_log_path = sim_log_path
        self.seed         = seed
        self.passed       = False
        self.conclusion   = ""
        self.duration     = duration
        self.duration_pct = -1
        self.num_errors   = -1
        self.num_warnings = -1
        self.set          = None
        self.group        = None
        self.args         = []
        self.errors       = []
        self.warnings     = []
        self.has_fatal    = False
        self.fatal        = ""
        self.notes        = []
    
    def add_arg(self, arg):
        self.args.append(arg)
    
    def add_error(self, text):
        self.errors.append(text)
    
    def add_warning(self, text):
        self.warnings.append(text)
    
    def add_note(self, text):
        self.notes.append(text)
    
    def render(self):
        self.num_errors   = len(self.errors  )
        self.num_warnings = len(self.warnings)


def interactive(ip_str, filename=""):
    vendor, name = common.parse_dep(ip_str)
    if vendor == "":
        ip = cache.get_anon_ip(name, True)
    else:
        ip = cache.get_ip(vendor, name, True)
    ip_str = f"{ip.vendor}/{ip.name}"
    timestamp = common.timestamp()
    results = Results(ip, filename, False, timestamp)
    test_set = TestSet("interactive")
    default_test_group = TestGroup("default")
    results.add_set(test_set)
    test_set.add_group(default_test_group)
    
    if ip_str in cfg.job_history:
        common.dbg(f"Parsing results for '{ip_str}'")
        if 'simulation' in cfg.job_history[ip_str]:
            for sim in cfg.job_history[ip_str]['simulation']:
                common.dbg("sim job history entry:\n" + str(sim))
                if (sim['type'] == "end") and (sim['is_regression'] == False):
                    sim_log_path = sim['log_path']
                    start = datetime.strptime(sim['timestamp_start'], "%Y/%m/%d-%H:%M:%S")
                    end   = datetime.strptime(sim['timestamp_end'  ], "%Y/%m/%d-%H:%M:%S")
                    duration = end - start
                    duration = divmod(duration.seconds, 60)[1]
                    new_test = Test(sim['test_name'], sim["dir_name"], sim["path"], sim_log_path, sim['seed'], duration)
                    
                    new_test.target_name = sim['target_name']
                    if sim['args'] != None:
                        for arg in sim['args']:
                            new_test.add_arg(arg)
                    new_test = parse_sim_log(results, sim_log_path, new_test)
                    if new_test.target_name == "default":
                        default_test_group.add_test(new_test)
                    else:
                        if new_test.target_name in test_set.groups:
                            test_set.groups[new_test.target_name].add_test(new_test)
                        else:
                            test_group = TestGroup(new_test.target_name)
                            test_set.add_group(test_group)
                            test_group.add_test(new_test)
    else:
        common.fatal(f"No simulation history for '{ip_str}'")
    
    results.render()
    if results.num_tests == 0:
        common.fatal("Did not find any test results")
    xml_file_path  = f"{cfg.sim_dir}/{filename}.xml"
    html_file_path = f"{cfg.sim_dir}/{filename}.html"
    gen_junit_xml(xml_file_path , results)
    gen_interactive_html_report(html_file_path, results)
    return results


def regression(input_filepath=""):
    tests_yml = None
    path = ""
    ip = None
    try:
        with open(input_filepath, 'r') as yamlfile:
            tests_yml = yaml.load(yamlfile, Loader=SafeLoader)
            if not tests_yml:
                raise Exception(f"tests.yml is empty")
            else:
                vendor = tests_yml['vendor']
                name   = tests_yml['ip']
                if vendor == "":
                    ip = cache.get_anon_ip(name, True)
                else:
                    ip = cache.get_ip(vendor, name, True)
                ip_str = f"{ip.vendor}/{ip.name}"
                filename = f""
                path = tests_yml['path']
                results = Results(ip, tests_yml['regression_name'], True, tests_yml['timestamp_start'])
                results.regression_name            = tests_yml['regression_name']
                results.regression_timestamp_start = tests_yml['timestamp_start']
                results.regression_timestamp_end   = tests_yml['timestamp_end']
                results.target_name                = tests_yml['target_name']
                results.test_suite_name            = tests_yml['test_suite_name']
                results.simulator                  = tests_yml['simulator']
                
                if results.target_name != "default":
                    results.has_target = True
                
                for test in tests_yml['tests']:
                    td = tests_yml['tests'][test]
                    start = datetime.strptime(td['timestamp_start'], "%Y/%m/%d-%H:%M:%S")
                    end   = datetime.strptime(td['timestamp_end'  ], "%Y/%m/%d-%H:%M:%S")
                    duration = end - start
                    duration = divmod(duration.seconds, 60)[1]
                    new_test = Test(td['name'], td['results_dir_name'], td['results_path'], td['sim_log_file_path'], td['seed'], duration)
                    for arg in td['args']:
                        new_test.add_arg(arg)
                    new_test = parse_sim_log(results, td['sim_log_file_path'], new_test)
                    set = td['set']
                    group = td['test_group']
                    if set not in results.sets:
                        new_set = TestSet(set)
                        results.add_set(new_set)
                    if group not in results.sets[set].groups:
                        new_group = TestGroup(group)
                        results.sets[set].add_group(new_group)
                    results.sets[set].groups[group].add_test(new_test)
    except Exception as e:
        common.fatal(f"Failed to parse regression results: {e}")
    
    results.render()
    if results.num_tests == 0:
        common.fatal("Did not find any test results")
    
    if results.has_target:
        filename = f"{ip.name}_{results.target_name}_{results.regression_name}"
    else:
        filename = f"{ip.name}_{results.regression_name}"
    
    xml_file_path  = f"{path}/{filename}.xml"
    html_file_path = f"{path}/{filename}.html"
    gen_junit_xml(xml_file_path , results)
    gen_regression_html_report(html_file_path, results)
    return results


def parse_sim_log(results, sim_log_path, test):
    sim_str = common.get_simulator_short_name(results.simulator)
    if not os.path.exists(sim_log_path):
        test.fatal = True
        test.passed = False
        test.add_note("No simulation log file")
        test.conclusion = "NO LOG!"
        return test
    
    for i, line in enumerate(open(sim_log_path)):
        for regex in cfg.warning_regexes:
            matches = re.search(regex, line)
            if matches:
                test.add_warning(matches.group(0))
        for regex in cfg.error_regexes:
            matches = re.search(regex, line)
            if matches:
                test.add_error(matches.group(0))
                test.passed = False
                test.conclusion = "FAILED"
        for regex in cfg.fatal_regexes:
            matches = re.search(regex, line)
            if matches:
                test.has_fatal = True
                test.fatal = matches.group(0)
                test.passed = False
                test.add_note("Has simulation fatal error")
                test.conclusion = "FATAL"
                break
        if sim_str == common.simulators_enum.VIVADO:
            for string in cfg.viv_fatal_errors:
                if string in line:
                    test.has_fatal = True
                    test.fatal = line
                    test.passed = False
                    test.add_note("Has vivado fatal error")
                    test.conclusion = "FATAL"
                    break
        if sim_str == common.simulators_enum.METRICS:
            for string in cfg.mdc_fatal_errors:
                if matches:
                    test.has_fatal = True
                    test.fatal = line
                    test.passed = False
                    test.add_note("Has dsim fatal error")
                    test.conclusion = "FATAL"
                    break
        if sim_str == common.simulators_enum.VCS:
            for string in cfg.vcs_fatal_errors:
                if matches:
                    test.has_fatal = True
                    test.fatal = line
                    test.passed = False
                    test.add_note("Has vcs fatal error")
                    test.conclusion = "FATAL"
                    break
        if sim_str == common.simulators_enum.QUESTA:
            for string in cfg.qst_fatal_errors:
                if matches:
                    test.has_fatal = True
                    test.fatal = line
                    test.passed = False
                    test.add_note("Has questa fatal error")
                    test.conclusion = "FATAL"
                    break
        if sim_str == common.simulators_enum.XCELIUM:
            for string in cfg.xcl_fatal_errors:
                if matches:
                    test.has_fatal = True
                    test.fatal = line
                    test.passed = False
                    test.add_note("Has xcelium fatal error")
                    test.conclusion = "FATAL"
                    break
        if sim_str == common.simulators_enum.RIVIERA:
            for string in cfg.riv_fatal_errors:
                if matches:
                    test.has_fatal = True
                    test.fatal = line
                    test.passed = False
                    test.add_note("Has riviera-pro fatal error")
                    test.conclusion = "FATAL"
                    break
    if test.conclusion != "FATAL":
        if len(test.errors) > 0:
            test.conclusion = "FAILED"
        else:
            test.conclusion = "PASSED"
            test.passed = True
    return test


def gen_junit_xml(filepath, results):
    group_count = 0
    testsuites = ET.Element("testsuites")
    testsuites.set('id', f"{results.ip_str}_{results.name}")
    testsuites.set('name', results.ip_str)
    testsuites.set('tests', str(results.num_tests))
    testsuites.set('failures', str(results.num_failed_tests))
    if results.is_regression:
        testsuites.set('time', str(results.regression_duration))
    else:
        testsuites.set('time', str(results.duration))
    
    for set in results.sets:
        cur_set = results.sets[set]
        for group in cur_set.groups:
            cur_group = cur_set.groups[group]
            testsuite = ET.SubElement(testsuites, "testsuite")
            testsuite.set('id', str(group_count))
            testsuite.set('name', f"{cur_set.name}.{cur_group.name}")
            testsuite.set('tests', str(cur_group.num_tests))
            testsuite.set('failures', str(cur_group.num_failed_tests))
            testsuite.set('time', str(cur_group.duration))
            for test in cur_group.failing_tests:
                testcase = ET.SubElement(testsuite, "testcase")
                testcase.set('id', test.results_dir)
                testcase.set('name', test.results_dir)
                testcase.set('time', str(test.duration))
                for error in test.errors:
                    failure = ET.SubElement(testcase, "failure")
                    failure.set("message", error)
                    failure.set("type", "ERROR")
                if test.has_fatal:
                    failure = ET.SubElement(testcase, "failure")
                    failure.set("message", test.fatal)
                    failure.set("type", "FATAL")
            for test in cur_group.passing_tests:
                testcase = ET.SubElement(testsuite, "testcase")
                testcase.set('id', test.results_dir)
                testcase.set('name', test.results_dir)
                testcase.set('time', str(test.duration))
            group_count += 1
    tree = ET.ElementTree(testsuites)
    try:
        tree.write(filepath)
    except Exception as e:
        common.fatal("Failed to write xml report to disk: " + str(e))
    common.dbg(f"Wrote {filepath}")
    results.xml_report_path = filepath


def gen_interactive_html_report(filepath, results):
    try:
        fin = open(interactive_html_report_template_path, "r")
        template_data = fin.read()
        html_report_template = Template(template_data)
        html_report_contents = html_report_template.render(results=results)
        with open(filepath,'w') as htmlfile:
            htmlfile.write(html_report_contents)
        htmlfile.close()
        common.dbg(f"Wrote {filepath}")
        results.html_report_path = filepath
    except Exception as e:
        common.fatal("Failed to write html report to disk: " + str(e))


def gen_regression_html_report(filepath, results):
    try:
        fin = open(regression_html_report_template_path, "r")
        template_data = fin.read()
        html_report_template = Template(template_data)
        html_report_contents = html_report_template.render(results=results)
        with open(filepath,'w') as htmlfile:
            htmlfile.write(html_report_contents)
        htmlfile.close()
        common.dbg(f"Wrote {filepath}")
        results.html_report_path = filepath
    except Exception as e:
        common.fatal("Failed to write html report to disk: " + str(e))



#!/usr/bin/python3
########################################################################################################################
# Copyright 2021-2023 Datum Technology Corporation
# All rights reserved.
########################################################################################################################


########################################################################################################################
# IMPORTS
########################################################################################################################
from datetime import date
import os
import sys
import re
import jinja2
from jinja2 import Template
from mio import cfg
from mio import common


########################################################################################################################
# GLOBALS
########################################################################################################################
dbg = True
args = {}
relative_path_to_template = "sets"
out_path = ""
default_license = "All rights reserved."
name_of_copyright_owner = ""
name = ""
name_normal_case = ""
answers_file = None
parameters = {}
default_copyright_owner = ""
default_vendor          = ""


########################################################################################################################
# MAIN
########################################################################################################################
def pick_out_path():
    global args
    global out_path
    global default_copyright_owner
    if (args['-o']):
        out_path = args['-o']
        print("Code will be output to " + out_path)
    if (args['-c']):
        default_copyright_owner = args['<copyright_owner>']
        print("Default copyright owner is " + default_copyright_owner)
        


def get_next_answer(question):
    #global args
    #global answers_file
    #if args['-f']:
    #    return answers_file.readline()
    #else:
    #    return input(question)
    return common.prompt(question)


def gen_component():
    global out_path
    global parameters
    parameters = {
        "year" : date.today().year
    }
    common.info("Moore.io Template Generator: Component (v1p0)")
    
    if out_path == "":
        out_path = get_next_answer("Please enter the destination path for this new Component (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    parameters["name_of_copyright_owner"] = name_of_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    parameters["vendor"] = name_of_vendor
    
    license = get_next_answer("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    parameters["license"] = license
    
    name = get_next_answer("Please enter the name for this Component (ex: 'my_comp'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["name"] = name
        parameters["name_uppercase"] = name.upper()
    
    pkg_name = get_next_answer("Please enter the package name for this Component (ex: 'uvma_apb'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["pkg_name"] = pkg_name
        parameters["pkg_name_uppercase"] = pkg_name.upper()
    
    base_class = get_next_answer("Please enter the base class type for this Component (default: 'uvm_component'):\n").lower().strip()
    if base_class == "":
        parameters["base_class"] = "uvm_component"
    else:
        parameters["base_class"] = base_class
    
    #process_file("Component", parameters, files['component']['in'], files['component']['out'])


def gen_object():
    global out_path
    global parameters
    parameters = {
        "year" : date.today().year
    }
    common.info("Moore.io Template Generator: Object (v1p0)")
    
    if out_path == "":
        out_path = get_next_answer("Please enter the destination path for this new Object (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    parameters["name_of_copyright_owner"] = name_of_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    parameters["vendor"] = name_of_vendor
    
    license = get_next_answer("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    parameters["license"] = license
    
    name = get_next_answer("Please enter the name for this Object (ex: 'my_obj'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["name"] = name
        parameters["name_uppercase"] = name.upper()
    
    pkg_name = get_next_answer("Please enter the package name for this Object (ex: 'uvma_apb'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["pkg_name"] = pkg_name
        parameters["pkg_name_uppercase"] = pkg_name.upper()
    
    base_class = get_next_answer("Please enter the base class type for this Object (default: 'uvm_object'):\n").lower().strip()
    if base_class == "":
        parameters["base_class"] = "uvm_object"
    else:
        parameters["base_class"] = base_class
    
    #process_file("Object", parameters, files['object']['in'], files['object']['out'])


def gen_reg_block():
    global out_path
    global parameters
    parameters = {
        "year" : date.today().year
    }
    common.info("Moore.io Template Generator: Register Block (v1p0)")
    
    if out_path == "":
        out_path = get_next_answer("Please enter the destination path for this new Register Block (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    parameters["name_of_copyright_owner"] = name_of_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    parameters["vendor"] = name_of_vendor
    
    license = get_next_answer("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    parameters["license"] = license
    
    name = get_next_answer("Please enter the name for this Register Block (ex: 'top'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["name"] = name
        parameters["name_uppercase"] = name.upper()
    
    pkg_name = get_next_answer("Please enter the package name for this Register Block (ex: 'uvme_my_ss'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["pkg_name"] = pkg_name
        parameters["pkg_name_uppercase"] = pkg_name.upper()
    
    base_class = get_next_answer("Please enter the base class type for this Register Block (default: 'uvm_reg_block'):\n").lower().strip()
    if base_class == "":
        parameters["base_class"] = "uvm_reg_block"
    else:
        parameters["base_class"] = base_class
    
    #process_file("Register Block", parameters, files['reg_block']['in'], files['reg_block']['out'])


def gen_reg():
    global out_path
    global parameters
    parameters = {
        "year" : date.today().year
    }
    common.info("Moore.io Template Generator: Register (v1p0)")
    
    if out_path == "":
        out_path = get_next_answer("Please enter the destination path for this new Register (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    parameters["name_of_copyright_owner"] = name_of_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    parameters["vendor"] = name_of_vendor
    
    license = get_next_answer("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    parameters["license"] = license
    
    name = get_next_answer("Please enter the name for this Register (ex: 'status'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["name"] = name
        parameters["name_uppercase"] = name.upper()
    
    pkg_name = get_next_answer("Please enter the package name for this Register (ex: 'uvme_my_ss'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["pkg_name"] = pkg_name
        parameters["pkg_name_uppercase"] = pkg_name.upper()
    
    base_class = get_next_answer("Please enter the base class type for this Register (default: 'uvm_reg'):\n").lower().strip()
    if base_class == "":
        parameters["base_class"] = "uvm_reg"
    else:
        parameters["base_class"] = base_class
    
    #process_file("Register", parameters, files['reg']['in'], files['reg']['out'])


def gen_test():
    global out_path
    global parameters
    parameters = {
        "year" : date.today().year
    }
    common.info("Moore.io Template Generator: Test (v1p0)")
    
    if out_path == "":
        out_path = get_next_answer("Please enter the destination path for this new Test (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    parameters["name_of_copyright_owner"] = name_of_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    parameters["vendor"] = name_of_vendor
    
    license = get_next_answer("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    parameters["license"] = license
    
    name = get_next_answer("Please enter the name for this Test (ex: 'smoke'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["name"] = name
        parameters["name_uppercase"] = name.upper()
    
    vseq_name = get_next_answer("Please enter the Virtual Sequence name for this Test (ex: 'basic_access'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["vseq_name"] = vseq_name
    
    pkg_name = get_next_answer("Please enter the package name for this Test (ex: 'my_ss'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["tb_name"] = pkg_name
        parameters["tb_name_uppercase"] = pkg_name.upper()
    
    base_class = get_next_answer("Please enter the base class type for this Test (ex: 'uvm_test'):\n").lower().strip()
    if base_class == "":
        common.fatal("ERROR: base class type cannot be empty.  Exiting.")
    else:
        parameters["base_class"] = base_class
    
    #process_file("Test", parameters, files['test']['in'], files['test']['out'])


def gen_vseq():
    global out_path
    global parameters
    parameters = {
        "year" : date.today().year
    }
    common.info("Moore.io Template Generator: Virtual Sequence (v1p0)")
    
    if out_path == "":
        out_path = get_next_answer("Please enter the destination path for this new Virtual Sequence (default: '.'):\n").strip()
        if out_path == "":
            out_path = "."
    
    
    name_of_copyright_owner = common.prompt("Please enter the name of the copyright holder or hit RETURN for the default (default is '" + default_copyright_owner + "'):\n").strip()
    if name_of_copyright_owner == "":
        name_of_copyright_owner = default_copyright_owner
    parameters["name_of_copyright_owner"] = name_of_copyright_owner
    
    name_of_vendor = common.prompt("Please enter a vendor name or hit RETURN for the default (default is '" + default_vendor + "'):\n").strip()
    if name_of_vendor == "":
        name_of_vendor = default_vendor
    parameters["vendor"] = name_of_vendor
    
    license = get_next_answer("Please enter a usage license or hit RETURN for the default (default is '" + default_license + "'):\n").strip()
    if license == "":
        license = default_license
    parameters["license"] = license
    
    name = get_next_answer("Please enter the name for this Virtual Sequence (ex: 'basic_access'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["name"] = name
        parameters["name_uppercase"] = name.upper()
    
    pkg_name = get_next_answer("Please enter the package name for this Virtual Sequence (ex: 'uvme_my_ss'):\n").lower().strip()
    if name == "":
        common.fatal("ERROR: name cannot be empty.  Exiting.")
    else:
        parameters["pkg_name"] = pkg_name
        parameters["pkg_name_uppercase"] = pkg_name.upper()
    
    base_class = get_next_answer("Please enter the base class type for this Virtual Sequence (ex: 'uvme_my_ss_base_vseq_c'):\n").lower().strip()
    if base_class == "":
        common.fatal("ERROR: base class type cannot be empty.  Exiting.")
    else:
        parameters["base_class"] = base_class
    
    #process_file("Virtual Sequence", parameters, files['vseq']['in'], files['vseq']['out'])



########################################################################################################################
# ENTRY POINT
########################################################################################################################
def main(template_name):
    global out_path
    global parameters
    global default_copyright_owner
    global default_vendor
    #if args['-f']:
    #    answers_file = open(args['-f'], "rt")
    
    #pick_out_path()
    
    default_copyright_owner = cfg.org_full_name
    default_vendor          = cfg.org_name
    
    global relative_path_to_template
    relative_path_to_template = cfg.mio_template_dir + "/sets/singleton/"
    
    common.fatal("Not yet implemented.")
    
    if (template_name == 'component'):
        gen_component()
    
    if (template_name == 'object'):
        gen_object()
    
    if (template_name == 'reg'):
        gen_reg()
    
    if (template_name == 'reg_block'):
        gen_reg_block()
    
    if (template_name == 'test'):
        gen_test()
    
    if (template_name == 'vseq'):
        gen_vseq()

